#include "tailfile.h"

using namespace std;

#define MEASURE(F,V,ITS) numtries = ITS; \
	start = finish = clock(); \
	while (finish-start < 250) { \
		start = finish = clock(); \
		for (long i=0;i<numtries;i++) { \
			F; \
		} \
		finish = clock(); \
		numtries = numtries*2; \
	} \
	cout << "Time=" << (double)(finish - start) * 1000 / ((numtries/2) * CLOCKS_PER_SEC); \
	cout << " ms (" << (numtries/2) << " tries)" << endl; \
	V = double(finish-start) * 1000/((numtries/2)* CLOCKS_PER_SEC);

unsigned long calcSize(long lng) {
	long ans = (long)ceil(((double)lng)/((double)BASELEN));
	if (ans == 0) {
		return 1;
	}
	return ans;
}

void readBigInt(string sa, BigInt* ba) {
	unsigned long sza = calcSize((long)sa.length());
	InitializeBigInt(ba,sza);
	for (unsigned int i=0;i<sza;i++) {
		long tmp = 0;
		long k = (long)sa.length()-BASELEN*i-BASELEN;
		for (unsigned int j=0;j<BASELEN;j++) {
			if (k>=0) {
				tmp = 10*tmp + (sa[k]-'0');
			}
			k++;
		}
		ba->Coef[i]=tmp;
	}
	ba->Size=sza;
}

void readBigInts(int tries) {
	
	string sa,sb;
	cin >> sa;
	cin >> sb;
	unsigned long sza = calcSize((long)sa.length());
	unsigned long szb = calcSize((long)sb.length());
	long ls = sza+szb;
	long n = 2;
	while (n < ls) {
		n = n*2;
	}
	InitializeFFT(n);

	BigInt* ba = new BigInt;
	readBigInt(sa,ba);

	BigInt* bb = new BigInt;
	readBigInt(sb,bb);

	BigInt* bc = new BigInt;
	InitializeBigInt(bc,ls+1);

	BigInt* bd = new BigInt;
	InitializeBigInt(bd,ls+1);

	clock_t start,finish;
	long numtries;
	double quadmul,logmul;
	
	cout << "fft mul time per multop: ";
	MEASURE(MulBigInt(ba,bb,bc),logmul,(tries));
	
	cout << "normal mul time per multop: ";
	MEASURE(MulBigIntNoFFT(ba,bb,bd),quadmul,(tries));

	cout << "lengths: " << ba->Size << " " << bb->Size << " " << bc->Size << endl;
	//PrintBigInt(ba);
	//PrintBigInt(bb);
	//PrintBigInt(bc);

	//PrintBigInt(bd);

	cout << "compare: " << CompareBigInt(bc,bd) << endl;
	cout << "fft muls performed 1 : " << logmul/quadmul << " normal mul's performed" << endl;
	
	cout << "fft worst square error: " << FFTSquareWorstError << endl;

	FreeBigInt(bd);
	delete bd;
	FreeBigInt(bc);
	delete bc;
	FreeBigInt(bb);
	delete bb;
	FreeBigInt(ba);
	delete ba;
	
}

int main( int argc , char *argv[ ]) {

	cin >> dec;
	cout << dec;
	
	int tries = 1;
	if (argc > 1) {
		tries = atoi(argv[1]);
		if (tries <= 0) {
			tries = 1;
		}
	}

	readBigInts(tries);
	return 0;

}